/*
  binforge - Repository system for Windows
  Copyright (C) 2009 Younes Serraj  Tous droits rservs.

  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la GNU General Public License telle que
  publie par la Free Software Foundation : soit la version 3 de cette
  licence, soit ( votre gr) toute version ultrieure.

  Ce programme est distribu dans lespoir quil vous sera utile, mais SANS
  AUCUNE GARANTIE : sans mme la garantie implicite de COMMERCIALISABILIT
  ni dADQUATION  UN OBJECTIF PARTICULIER. Consultez la Licence Gnrale
  Publique GNU pour plus de dtails.

  Vous devriez avoir reu une copie de la Licence Gnrale Publique GNU avec
  ce programme ; si ce nest pas le cas, consultez :
  <http://www.gnu.org/licenses/>.
*/

#ifdef BINFORGE_H
#ifndef SOCKET_C
#define SOCKET_C

#include <stdio.h>

int sck_init() {
	WSADATA WSAData;
    return WSAStartup(MAKEWORD(2,0), &WSAData);
}

void sck_clear() {
	WSACleanup();
}

int sck_makeSocket() {
	int s;

	s = socket(AF_INET, SOCK_STREAM, 0);
	if (s == INVALID_SOCKET) {
#ifdef VERBOSE
        printf("VERBOSE: sck_makeSocket() failed.\n");
#endif
	    return 0;
	}
 	return s;
}

int sck_connect(char *hostname, int port) {
	int s;
	struct sockaddr_in server;

	if ((s = sck_makeSocket()) == INVALID_SOCKET)
	    return 0;

	server.sin_family = AF_INET;

	if((long)inet_addr(hostname) != (long)-1) // Si l'adresse communique est une adresse ip
        server.sin_addr.s_addr = inet_addr(hostname);
    else
    { // tentative de rsolution de DNS
        struct hostent *serverip;
        serverip = gethostbyname(hostname);
        if(!serverip)
            return 0;
        char* IP;
        memcpy(&(server.sin_addr), serverip->h_addr, 0);
        memcpy(&(server.sin_addr), serverip->h_addr, serverip->h_length);
        IP = inet_ntoa(server.sin_addr);
        return sck_connect(IP, port);
    }

	server.sin_port = htons(port);
	memset(&server.sin_zero, '\0', sizeof(server.sin_zero));

	if (connect(s, (struct sockaddr*)&server, sizeof(server)) == SOCKET_ERROR){
	    if (WSAGetLastError()==WSAECONNREFUSED)
            printf("ESM_SOCKERR_CONNECTION_REFUSED\n");
        else
            printf("ESM_SOCKERR_CANNOT_CONNECT\n");
	    return 0;}

	return s;
}

void sck_listen(int port, void(*callback)(struct callback_params*), int multithread) {
	int s;
	struct sockaddr_in server;
	struct callback_params *cp;
	int csize;

	if ((s = sck_makeSocket()) == INVALID_SOCKET)
	    return;

	csize = sizeof(cp->s_in);

	server.sin_family = AF_INET;
	server.sin_addr.s_addr = htonl(INADDR_ANY);
	server.sin_port = htons(port);
	memset(&server.sin_zero, '\0', sizeof(server.sin_zero));

	if (bind(s, (SOCKADDR *)&server, sizeof(server)) == SOCKET_ERROR)
        return;
    if (listen(s, 0) == SOCKET_ERROR) /* listen : commencer l'ecoute */
        return;

    while (s) {
        cp = (struct callback_params*)malloc(sizeof(struct callback_params));
        if (cp == NULL)
        	return;

        cp->s = accept(s, (SOCKADDR *)&(cp->s_in), &csize);

        if (cp->s == INVALID_SOCKET) {
        	free(cp);
        	continue;
        }
        if (multithread == true)
			thread_create(callback, cp);
		else {
			callback(cp);
			return;
		}
    }

	return;
}

char *sck_read(int s, int maxlen) {
	char *buffer;

	if (s == INVALID_SOCKET)
        return (char*)NULL;

	buffer = (char*)malloc(sizeof(char)*(maxlen+1));
	if (buffer == NULL)
		return NULL;

    memset(buffer, '\0', sizeof(buffer));

	if ((int)recv(s, buffer, sizeof(buffer), 0) < 1) {
		free(buffer);
		return (char*)NULL;
	}
	return buffer;
}

char *sck_readln(int s, int maxlen) {
	char *buffer;
	char c;
	int i=0;
	int recv_ret;

	buffer = (char*)malloc(sizeof(char)*(maxlen+1));
	if (buffer == NULL)
		return NULL;

    memset(buffer, '\0', sizeof(buffer));

	while (i<maxlen) {
		recv_ret = (int)recv(s, &c, sizeof(char), 0);

		if (recv_ret == SOCKET_ERROR) { // error while reading
			if (i == 0) {
				free(buffer);
				return (char*)NULL;
			}
			break;
		}
		else if (recv_ret == 0) // no data to read
			break;
		if (c == '\n' || c == '\r') // end of line reached
			break;
		buffer[i] = c; // copy char read into buffer
		i++;
	}

    if (i == 0) {
        free(buffer);
        return (char*)NULL;
    }

    buffer[i] = '\0';
	return buffer;
}

int *sck_readfile(int s, int size, FILE *fp) {
	char c;
	int i=0;
	int recv_ret;

	if (fp == NULL)
        return false;

    rewind(fp);

	while (i<size) {
		recv_ret = (int)recv(s, &c, sizeof(char), 0);

		if (recv_ret == SOCKET_ERROR) { // error while reading
			break;
		}
		else if (recv_ret == 0) // no data to read
			break;
		fputc(c, fp);
		i++;
	}

	return i;
}

int sck_send(int s, char *data) {
	return (int)send(s, data, strlen(data), 0);
}

int sck_sendln(int s, char *data) {
	char *buffer;
	int ret;

	if (strlen(data) == 0)
	    return sck_send(s, END_OF_LINE);

	buffer = (char*)malloc(sizeof(char)*(strlen(data)+strlen(END_OF_LINE)));
	if (buffer == NULL)
		return 0;

	sprintf(buffer, "%s%s", data, END_OF_LINE);
	ret = sck_send(s, buffer);
	free(buffer);

	return ret;
}

int sck_close(int s) {
	closesocket(s);
}

#endif SOCKET_C
#endif BINFORGE_H
